<?php
/*
 * Marco's Google(TM) bot access for registered users' pages
 * A plugin that allows access for indexing registered users' pages to spiders and bots of major search engines.
 * Plugin for Joomla 2.5 - Version 1.0.0
 * License: http://www.gnu.org/copyleft/gpl.html
 * Authors: marco maria leoni
 * Copyright (c) 2013 marco maria leoni web consulting - http: www.mmleoni.net
 * Project page at http://www.mmleoni.net/search-engine-access-to-registered-users-pages
 * *** Last update: Sep 16th, 2013 ***
*/
defined( '_JEXEC' ) or die( 'Restricted access' );
jimport( 'joomla.plugin.plugin' );
class plgSystemMarcosbotaccess extends JPlugin{
	public function plgSystemMarcosbotaccess( &$subject, $config ){
		parent::__construct( $subject, $config );
	}

	public function onAfterInitialise(){
		$app = JFactory::getApplication();

		if ($app->isAdmin()) return;
		$p_debugMode = $this->params->get('debugMode', 1);
		
		$p_searchBotsAndUser = array();
		foreach( preg_split('/\r\n?/', $this->params->get('searchBotsAndUser', '')) as $line){
			list($ua,$u) = explode ('|', $line);
			if (!trim($ua)) continue;
			//JFactory::getApplication()->enqueueMessage( "user agent: {$ua}  -- user {$u}", 'notice' );
			$p_searchBotsAndUser[$ua] = $u;
		}

		// check if user is logged
		$user = JFactory::getUser();
		if ($user->id) {
			if($p_debugMode) JFactory::getApplication()->enqueueMessage( "You are logged in as User name: {$user->username }, Real name: {$user->name}, User ID  : {$user->id}", 'message');
			foreach($p_searchBotsAndUser as $u){
				if (JString::strcmp($user->username, $u)==0){
					// if user is a bot send noarchive...
					$this->noCacheHeader();
					break;
				}
			}
			return;  // ...nothing else to do!
		}
		
		
		$p_userPassword=$this->params->get('userPassword', '');
		$p_useGenericUser = $this->params->get('useGenericUser', 1);
		$p_checkIfUserExists = $this->params->get('checkIfUserExists', 1);
		
		$p_sendNotification = $this->params->get('sendnotification',0);
		$p_sendTo = $this->params->get('sendto','');
		$p_sendFrom = $app->getCfg('mailfrom');
		if(!$p_sendTo) $p_sendTo = $p_sendFrom ;
		
		$p_authOnIP = $this->params->get('authOnIP', 0);

		$p_ipList = $this->params->get('ipList', '');
		$p_debugMode = $this->params->get('debugMode', 1);
		$remoteIP = ip2long($_SERVER['REMOTE_ADDR']);


		$userName = '';
		$msg = array();
		$userAgent = $_SERVER['HTTP_USER_AGENT'];
		$found = FALSE;
		foreach($p_searchBotsAndUser as $ua=>$u){
			if (JString::strpos($userAgent, $ua)===FALSE) continue;
			$found = TRUE;
			$userName = $u;
			break;
		}
		if (!$found) return; // not an allowed search engine

		// Search engine bot detected
		
		// first: instruct the search engine to not cache the page, only index it.
		$this->noCacheHeader();
		
		if ($p_useGenericUser) $userName = 'SEGenericBot';
		$msg[] = "Search bot {$userAgent} detected, selected {$userName} user.";
		
		if ($p_checkIfUserExists){
			//$userCheck = JFactory::getUser($userName);
			// known bug in getUser, see http://joomlacode.org/gf/project/joomla/tracker/?action=TrackerItemEdit&tracker_item_id=29986
			// so I wrote my own function...
			$userCheck = $this->getUserByName($userName);
			if (!$userCheck->id) {
				$this->addJoomlaUser($userName, $p_userPassword);
				$msg[] = "Created user {$userName} for {$ua} bot.";
				if($p_debugMode) JFactory::getApplication()->enqueueMessage("Created user {$userName} for {$ua} bot." , 'notice');
			}
		}
		
		//$user = JFactory::getUser('SEGenericBot');
		$user = $this->getUserByName($userName);
		if (!$user->id){
			if($p_debugMode) JFactory::getApplication()->enqueueMessage("User {$user} not found." , 'error');
			return;
		}
		
		$credentials = array('username' => $user->username, 'password' => $p_userPassword);
		$options = array('remember' => TRUE);
		$msg[] = "Try logging as {$user->username}.";
		if($p_debugMode) JFactory::getApplication()->enqueueMessage("Try logging as {$user->username}." , 'message');
		if ($app->login($credentials, $options)){
			$msg[] = "Success logging as {$user->username}.";
		}else{
			$msg[] = "Error logging as {$user->username}.";
		}
		
		if(($p_sendNotification) AND ($msg)) $this->sendNotification($p_sendTo, $p_sendFrom, $msg);
	}
	
	protected function noCacheHeader(){
		// instruct the search engine to not cache the page, only index it.
		if ($this->params->get('setNoArchive', 1)){
			JFactory::getDocument()->setMetaData( 'robots', 'noarchive' );
		}
	}
	
	protected function sendNotification($to, $from, $msg){
		$app = JFactory::getApplication();
	
		$msg = implode("\r\n", $msg);
		$msg .= "\r\n\r\n";
		
		$mail = JFactory::getMailer();
		$mail->setSender($from);
		$mail->addRecipient($to);
		$mail->setSubject($app->getCfg('sitename') . ' Marco\'s Google bot access for registered Pages' );
		$mail->setBody($msg);
		$mail->isHTML(false);
		$mail->Send();		
	}
	
	
	protected function addJoomlaUser($username, $password) {
		jimport('joomla.user.helper');

		$user = JUser::getInstance();
		$user->set('id', 0);
		$user->set('name', $username);
		$user->set('username', $username);
		$user->set('usertype', 'deprecated');
		//$user->set('password_clear', $password);
		$salt = JUserHelper::genRandomPassword(32);
		$crypted  = JUserHelper::getCryptedPassword($password, $salt);
		$user->set('password', $crypted . ':' . $salt);
		$user->set('email', $username . '@example.com');
		$user->set('groups', array('2'));

		if (!$user->save()) {
		  JFactory::getApplication()->enqueueMessage("Could not save user. Error: " . JText::_($user->getError()), 'error');
		}
	}
	

	protected function getUserByName($username) {
		$db = JFactory::getDBO();
		
		$sql = "SELECT `id` from `#__users` WHERE `username` = " . $db->Quote($username) ;
		$db->setQuery( $sql );
		$db->query( $sql );
		if(!$id=$db->loadResult()){
			if($p_debugMode) JFactory::getApplication()->enqueueMessage("Could not found user {$username}." , 'notice');
		}
		return JFactory::getUser($id);
	}
	
}